import time

import click
import webcolors
from blink1.blink1 import Blink1


@click.command()
@click.option('--on', default='white', help='Color to flash on')
@click.option('--off', default='black', help='Color to flash off')
@click.option('--duration', default=1, help='Length of each flash cycle in seconds')
@click.option('--repeat', default=2, help='Number of times to flash')
@click.option('--fade', default=0.2, help='Fade time in seconds')
def flash(on, off, duration, repeat, fade):
    blink1 = Blink1()


    for i in range(0, repeat):
        blink1.fade_to_color(fade * 1000, on)
        time.sleep(duration)
        blink1.fade_to_color(fade * 1000, off)
        time.sleep(duration)

    blink1.fade_to_rgb(fade * 1000, 0, 0, 0)


if __name__ == '__main__':
    flash()



import unittest
import math
import mock
from blink1.blink1 import ColorCorrect


class TestGamma(unittest.TestCase):

    def testUnity(self):
        g = ColorCorrect(gamma=(1,1,1), white_point=(255,255,255))
        self.assertEquals(
            g(255,255,255),
            (255,255,255)
        )

    def testFullPowerHalfGamma(self):
        expected = round(255 * (1 ** 0.5))
        g = ColorCorrect(gamma=(0.5,0.5,0.5), white_point=(255,255,255))
        self.assertEquals(
            g(255,255,255),
            (expected,expected,expected)
        )

    def testHalfPowerHalfGamma(self):
        expected = round(255 * (127 / 255) ** 0.5)
        g = ColorCorrect(gamma=(0.5,0.5,0.5), white_point=(255,255,255))
        self.assertEquals(
            g(127,127,127),
            (expected,expected,expected)
        )

if __name__ == '__main__':
    unittest.main()


import click
from blink1.blink1 import blink1


@click.command()
@click.option('--color', default='white', help='What colour to set the Blink(1)')
@click.option('--white_point', default='incandescent', help='The name of the white-point to use')
def shine(color, white_point):
    with blink1(switch_off=False, white_point=white_point) as b1:
        b1.fade_to_color(100, color)


if __name__ == '__main__':
    flash()


#!/usr/bin/env python3

import hid

REPORT_ID = 1
VENDOR_ID = 0x27b8
PRODUCT_ID = 0x01ed

devs = hid.enumerate( VENDOR_ID, PRODUCT_ID)
#print(devs)
serials = []
for d in devs:
    print(d)
    serials.append(d.get('serial_number'))

serialz = map(lambda d:d.get('serial_number'), devs)

serial_number = None

#devs = map(lambda d: print(d), devs)
print("serials:", serials)
print("serialz:", serialz)

hidraw = hid.device( VENDOR_ID, PRODUCT_ID, serial_number )
hidraw.open( VENDOR_ID, PRODUCT_ID, serial_number )
#hidraw.send_feature_report([0x02, 0x10, 0x00,0x00,0x00,0x00,0x00,0x00])
featureReport = [REPORT_ID, 99, 255, 0, 255, 0, 11, 0, 0];
hidraw.send_feature_report( featureReport )



import unittest

import mock
from blink1.blink1 import Blink1, BlinkConnectionFailed, InvalidColor


class TestSimpleLightControl(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.b1 = Blink1()

    @classmethod
    def tearDownClass(cls):
        cls.b1.off()
        cls.b1.close()
        del cls.b1

    def testOn(self):
        self.b1.fade_to_color(1000, 'white')

    def testInvalidColor(self):
        with self.assertRaises(InvalidColor):
            self.b1.fade_to_color(1000, 'moomintrol')

    def testAlsoWhite(self):
        self.b1.fade_to_color(1000, (255,255,255))

    def testAWhiteShadeOfPale(self):
        self.b1.fade_to_color(1000, '#ffffff')

    def testAGreyerShadeOfPale(self):
        self.b1.fade_to_color(1000, '#eeeeee')

    def testAnImplausibleShadeOfWhite(self):
        with self.assertRaises(InvalidColor):
            self.b1.fade_to_color(1000, '#xxxxxx')

    def testOff(self):
        self.b1.off()

    def test_get_firmware_version(self):
        ver = self.b1.get_version()

    def test_get_serial_number(self):
        sn = self.b1.get_serial_number()


class TestFailedConnection(unittest.TestCase):
    def testCannotFind(self):
        with mock.patch('blink1.blink1.PRODUCT_ID', '0101'):
            with self.assertRaises(BlinkConnectionFailed):
                b1 = Blink1()


if __name__ == '__main__':
    unittest.main()


import unittest
import time

from blink1.blink1 import blink1
from blink1.kelvin import COLOR_TEMPERATURES


class TestBlink1ContextManager(unittest.TestCase):
    def test_cm(self):
        with blink1() as b1:
            b1.fade_to_color(0, "teal")

    def test_cm_closed(self):
        with blink1() as b1:
            b1.close()
            b1.fade_to_color(0, "teal")

    def test_cm_with_gamma(self):
        with blink1(gamma=(.5, .5, .5)) as b1:
            b1.fade_to_color(0, "teal")

    def test_cm_with_white_point(self):
        with blink1(white_point=(255, 255, 255)) as b1:
            b1.fade_to_color(0, "white")
            time.sleep(0.1)

    def test_cm_with_white_point(self):
        with blink1(white_point=COLOR_TEMPERATURES['candle']) as b1:
            b1.fade_to_color(0, "white")
            time.sleep(0.1)

    def test_cm_with_white_point(self):
        with blink1(white_point='blue-sky') as b1:
            b1.fade_to_color(0, "white")
            time.sleep(0.1)



if __name__ == '__main__':
    unittest.main()


from setuptools import setup
import os

PROJECT_ROOT, _ = os.path.split(__file__)
REVISION = '0.1.1'
PROJECT_NAME = 'blink1'
PROJECT_AUTHORS = "Salim Fadhley, Tod E. Kurt"
PROJECT_EMAILS = 'salimfadhley@gmail.com, todbotdotcom@gmail.com'
PROJECT_URL = "https://github.com/todbot/blink1-python"
SHORT_DESCRIPTION = 'Official blink(1) control library'

try:
    DESCRIPTION = open(os.path.join(PROJECT_ROOT, "README.md")).read()
except IOError:
    DESCRIPTION = SHORT_DESCRIPTION

setup(
    name=PROJECT_NAME.lower(),
    version=REVISION,
    author=PROJECT_AUTHORS,
    author_email=PROJECT_EMAILS,
    packages=['blink1', 'blink1_tests'],
    zip_safe=True,
    include_package_data=False,
    install_requires=['hidapi>=0.7.99', 'click', 'webcolors'],
    test_suite='nose.collector',
    tests_require=['mock', 'nose', 'coverage'],
    url=PROJECT_URL,
    description=SHORT_DESCRIPTION,
    long_description=DESCRIPTION,
    license='MIT',
    entry_points={
        'console_scripts': [
            'blink1-flash = blink1.flash:flash',
            'blink1-shine = blink1.shine:shine'
        ]
    },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Testing',
    ],
)



"""
blink1.py -- blink(1) Python library using python hidapi

All platforms:
 % pip install blink1

"""
import logging
import time
import sys
from contextlib import contextmanager
import webcolors
import hid
#from builtins import str as text

# from .kelvin import kelvin_to_rgb, COLOR_TEMPERATURES


class Blink1ConnectionFailed(RuntimeError):
    """Raised when we cannot connect to a Blink(1)
    """

class InvalidColor(ValueError):
    """Raised when the user requests an implausible colour
    """


log = logging.getLogger(__name__)
#logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.DEBUG)
#getattr(log, loglevel.upper())

DEFAULT_GAMMA = (2, 2, 2)
DEFAULT_WHITE_POINT = (255, 255, 255)

REPORT_ID = 0x01
VENDOR_ID = 0x27b8
PRODUCT_ID = 0x01ed

class ColorCorrect(object):
    """Apply a gamma correction to any selected RGB color, see:
    http://en.wikipedia.org/wiki/Gamma_correction
    """
    def __init__(self, gamma, white_point):
        """
        :param gamma: Tuple of r,g,b gamma values
        :param white_point: White point expressed as (r,g,b), integer color temperature (in Kelvin) or a string value.

        All gamma values should be 0 > x >= 1
        """
        self.gamma = gamma

        if isinstance(white_point, str):
            kelvin = COLOR_TEMPERATURES[white_point]
            self.white_point = kelvin_to_rgb(kelvin)
        elif isinstance(white_point,(int,float)):
            self.white_point = kelvin_to_rgb(white_point)
        else:
            self.white_point = white_point

    @staticmethod
    def gamma_correct(gamma, white, luminance):
        return round(white * (luminance / 255) ** gamma)

    def __call__(self, r, g, b):
        color = [r,g,b]
        return tuple(self.gamma_correct(g, w, l) for (g, w, l) in zip(self.gamma, self.white_point, color) )


class Blink1:
    """Light controller class, sends messages to the blink(1) and blink(1) mk2 via USB HID.
    """
    def __init__(self, serial_number=None, gamma=None, white_point=None):
        """
        :param serial_number: serial number of blink(1) to open, otherwise first found
        :param gamma: Triple of gammas for each channel e.g. (2, 2, 2)
        """
        self.cc = ColorCorrect(
            gamma=(gamma or DEFAULT_GAMMA),
            white_point=(white_point or DEFAULT_WHITE_POINT)
        )
        self.dev = self.find(serial_number)

    def close(self):
        self.dev = None

    @staticmethod
    def find(serial_number=None):
        """
        Find a praticular blink(1) device, or the first one
        :param serial_number: serial number of blink(1) device (from Blink1.list())
        """
        try:
            hidraw = hid.device(VENDOR_ID,PRODUCT_ID,serial_number)
            hidraw.open(VENDOR_ID,PRODUCT_ID,serial_number)
            # hidraw = hid.device(VENDOR_ID,PRODUCT_ID,unicode(serial_number))
            # hidraw.open(VENDOR_ID,PRODUCT_ID,unicode(serial_number))
#        except IOError as e:
#            raise Blink1ConnectionFailed(e)
        except OSError as e:
            hidraw = None

        return hidraw

    @staticmethod
    def list():
        """
        List blink(1) devices connected, by serial number
        :return: List of blink(1) device serial numbers
        """
        try:
            devs = hid.enumerate(VENDOR_ID,PRODUCT_ID)
            serials = map(lambda d:d.get('serial_number'), devs)
            return serials
        except IOError as e:
            return []

    def notfound(self):
        return None  # fixme what to do here

    def write(self,buf):
        """
        Write command to blink(1)
        Send USB Feature Report 0x01 to blink(1) with 8-byte payload
        Note: arg 'buf' must be 8 bytes or bad things happen
        """
        log.debug("blink1write:" + ",".join('0x%02x' % v for v in buf))
        self.dev.send_feature_report(buf)

    def read(self):
        """
        Read command result from blink(1)
        Receive USB Feature Report 0x01 from blink(1) with 8-byte payload
        Note: buf must be 8 bytes or bad things happen
        """
        buf = self.dev.get_feature_report(REPORT_ID,9)
        log.debug("blink1read: " + ",".join('0x%02x' % v for v in buf))
        return buf

    def fade_to_rgb_uncorrected(self, fade_milliseconds, red, green, blue, led_number=0):
        """
        Command blink(1) to fade to RGB color, no color correction applied.
        """
        action = ord('c')
        fade_time = int(fade_milliseconds / 10)
        th = (fade_time & 0xff00) >> 8
        tl = fade_time & 0x00ff
        buf = [REPORT_ID, action, int(red), int(green), int(blue), th, tl, led_number, 0]
        self.write( buf )

    def fade_to_rgb(self,fade_milliseconds, red, green, blue, led_number=0):
        r, g, b = self.cc(red, green, blue)
        return self.fade_to_rgb_uncorrected(fade_milliseconds, r, g, b, led_number=0)

    @staticmethod
    def color_to_rgb(color):
        if isinstance(color, tuple):
            return color
        if color.startswith('#'):
            try:
                return webcolors.hex_to_rgb(color)
            except ValueError:
                raise InvalidColor(color)

        try:
            return webcolors.name_to_rgb(color)
        except ValueError:
            raise InvalidColor(color)


    def fade_to_color(self, fade_milliseconds, color):
        """
        Fade the light to a known colour in a
        :param fade_milliseconds: Duration of the fade in milliseconds
        :param color: Named color to fade to
        :return: None
        """
        red, green, blue = self.color_to_rgb(color)

        return self.fade_to_rgb(fade_milliseconds, red, green, blue)

    def off(self):
        """Switch the blink(1) off instantly
        """
        self.fade_to_color(0, 'black')

    def get_version(self):
        """Get blink(1) firmware version
        """
        if ( self.dev == None ): return ''
        buf = [REPORT_ID, ord('v'), 0, 0, 0, 0, 0, 0, 0]
        self.write(buf)
        time.sleep(.05)
        version_raw = self.read()
        version = (version_raw[3] - ord('0')) * 100 + (version_raw[4] - ord('0'))
        return str(version)

    def get_serial_number(self):
        """Get blink(1) serial number
        """
        return self.dev.get_serial_number_string()
        # return usb.util.get_string(self.dev, 256, 3)


@contextmanager
def blink1(switch_off=True, gamma=None, white_point=None):
    """Context manager which automatically shuts down the Blink(1)
    after use.
    """
    # b1 = Blink1(gamma=gamma, white_point=white_point)
    b1 = Blink1()
    yield b1
    if switch_off:
        b1.off()
    b1.close()



import unittest
import math
import mock
import time
from blink1.kelvin import kelvin_to_rgb
from blink1.blink1 import blink1


class TestGamma(unittest.TestCase):

    def test_red_6000(self):
        r, g, b = kelvin_to_rgb(6000)
        self.assertEquals(r, 255)

    def test_blue_6700(self):
        r, g, b = kelvin_to_rgb(6700)
        self.assertEquals(b, 255)

    def test_range(self):
        """Verify that it is possible to produce the entire color range"""

        for k in range(10, 12000, 400):
            r,g,b = kelvin_to_rgb(k)
            self.assertIsInstance(r, int)
            self.assertIsInstance(g, int)
            self.assertIsInstance(b, int)

            with blink1(white_point=(r,g,b)) as b1:
                b1.fade_to_color(0, 'white')
                time.sleep(0.05)


if __name__ == '__main__':
    unittest.main()